@@ -18,11 +18,11 @@ test_that("Multiple registration works", {
         skip_on_os("solaris")
         
         reg <- niftyreg(skewedColourHouse, house, symmetric=FALSE)
-        expect_that(forward(reg,2)[1,2], equals(0.1,tolerance=0.05))
+        expect_that(forward(reg)[1,2], equals(0.1,tolerance=0.05))
         
         skip_on_cran()
         
         reg <- niftyreg(skewedColourHouse, house, scope="nonlinear", init=forward(reg), symmetric=FALSE, maxIterations=300)
-        expect_that(dim(forward(reg,2)), equals(c(40L,56L,1L,1L,2L)))
+        expect_that(dim(forward(reg)), equals(c(40L,56L,1L,1L,2L)))
     }
 })
